import os
import logging


from socketIO_client import SocketIO, LoggingNamespace, BaseNamespace



from npwcli.spider import Spider





class NotepadSocket(SocketIO):

    def __init__(self, url_key):

        self.socket_url = 'https://live.notepad.pw'
        self.port = 443
        self.url_key = url_key
        self.filepath = ''
        self.file_stamp = 0


        # self.io = SocketIO(self.socket_url, self.port, BaseNamespace)

        super().__init__(self.socket_url, self.port, BaseNamespace)

        # logging.getLogger('socketIO-client').setLevel(logging.DEBUG)
        # logging.basicConfig()
        
     


    def join_room(self):
        self.emit('join_room', self.url_key)

    def publish(self, content):


        io_data = dict()

        io_data['name'] = self.url_key
        io_data['text'] = content
        io_data['cursor_location'] = len(content)-1

        self.emit('editing', io_data)

    
        
class Notepad:

    def __init__(self, url_key, live_update=False):

        self.url_key = url_key
        self.domain = "https://notepad.pw/"
        self.spider = Spider(self.domain+self.url_key)
        self.pad_key = self.spider.pad_key

        self.content = self.spider.content
        self.haspw = (self.spider.url_key == '' and self.spider.pad_key == '')

        self.io = None

        if live_update:
            self.io = NotepadSocket(self.url_key)
            self.io.join_room()


    def get_content_from_file_path(self, file_path):

        content = ''

        with open(file_path) as f:
            content = f.read()

        return content


    def is_file_content_changed(self):

        new_file_stamp = os.stat(self.filepath).st_mtime

        if(new_file_stamp != self.file_stamp):
            return True
        return False


    def save_to_file(self, filename, overwrite):

        with open(filename, 'w') as f:
            f.write(self.content)

        return


    def save_file(self, filepath, overwrite):

        file_content = self.get_content_from_file_path(filepath)

        self.filepath = filepath
        self.file_stamp = os.stat(filepath).st_mtime

        if(overwrite):
            self.content = file_content
        else:
            self.content += file_content


        if self.io:
            self.io.publish(self.content)


        data = dict()

        data['key'] = self.pad_key
        data['pad'] = self.content
        data['pw'] = ''
        data['url'] = self.url_key
        data['monospace'] = 0
        data['caret'] = 1

        self.spider.save(data)

        return

    def view_file(self):
        return self.content

np = None

if __name__ == '__main__':

    np = Notepad('chaityashah', True)


#!/usr/bin/env python3
"""
Notepad.pw CLI

Usage: 	
	npw [options] FILE LINK
	npw --get LINK FILE


Example:
	npw -lo test.py testingthiscli

Options:
	-h, --help		Show this screen
	--version       	Show version
	-l, --live-update   	Enable live update on notepad
	-o, --overwrite     	Overwrite notepad contents
	-w ,--watch         	Watch file for changes 
	-g, --get           	Copy contents of pad to a file

Note: Watch mode will overwrite contents of the notepad

"""

import os
import time

from docopt import docopt
from termcolor import cprint


from npwcli.update_content import Notepad
from npwcli import __version__




WARNING = 'red'
MESSAGE = 'blue'
SUCCESS = 'green'




def start():


	arguments = docopt(__doc__, version='npw version '+'.'.join(str(i) for i in __version__))
	

	filename = arguments.get('FILE', None)
	curr_dir = os.getcwd()
	file_path = os.path.join(curr_dir, filename)


	live_update = arguments.get('--live-update', False)
	watch = arguments.get('--watch', False)

	get= arguments.get('--get', False)



	link = arguments.get('LINK', None)


	try: 

		cprint('Connecting to notepad.pw....',MESSAGE)

		notepad = Notepad(link, live_update=live_update)

	except Exception as e:

		cprint("\nError: Something went wrong...", WARNING)
		return -1

	

	if(get):
		notepad.save_to_file(filename, True)
		cprint("Saved contests of {} to {} succesfully".format(link, filename), SUCCESS)
		return 1;




	if(notepad.haspw):
		cprint('\nPASS: The given url is password protected', MESSAGE)
	else:
		cprint("Saving {} to {}..... \n".format(filename,link), SUCCESS)
		notepad.save_file(file_path, arguments['--overwrite'])

		try:

			if watch:
				cprint('Watching {} for changes'.format(filename),MESSAGE)

			while watch:
				time.sleep(5)
				if notepad.is_file_content_changed():
					cprint('\nChanges detected', MESSAGE)
					notepad.save_file(file_path, True)
					cprint('Changes saved',MESSAGE)

		except  KeyboardInterrupt:

			cprint('\nClosing npw', MESSAGE) 


	






import re

def is_url(url):
	url_pattern = r'^https?://(.*?)\.(.*?)$'
	result = re.findall(url_pattern, url)

	if len(result) is 0: 
		return False
	return True

def make_url(url, ext):
	if ext[0] is '/':
		return url + ext
	else:
		return url + '/' + ext



import re

from urllib import request
from urllib.request import Request
from urllib.parse import urlencode
from multiprocessing.dummy import Pool as ThreadPool

from brotli import decompress

from npwcli.utils import is_url, make_url



class Spider:

    def __init__(self,url, verbose=False):
        self.url = url
        self.domain = ''
        self.response = ''
        self.verbose = verbose
        self.content = ''

        self.get_domain()
        self.USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36'

        self.save_url = 'https://notepad.pw/save'
        
        


        self.url_key = ""

        self.pad_key = ""
        self.visit()

        if(self.url_key != "" and self.pad_key != ""):
            self.hit("https://notepad.pw/fetch/"+self.url_key+"/?_=153264234352")

       
    
    def get_domain(self):

        pattern = r'(http[s]?://[a-zA-Z0-9]+?\.?[a-zA-Z0-9\-]+\.[a-z]{2,})'
        result = re.findall(pattern, self.url)

        if(result):
            self.domain = result[0]

    def visit(self):
            
            if(self.verbose):
                print("Visiting {}".format(self.url))
            
            try:
                request_obj = Request(self.url)
                request_obj.add_header('User-Agent',self.USER_AGENT)
                request_obj.add_header('Content-Type', 'text/html')
                response_obj = request.urlopen(request_obj)
                self.headers = response_obj.getheaders()
                self.cookies = response_obj.getheader('Set-Cookie')
                self.response = response_obj.read().decode('utf-8');
                
                self.get_pad_key(self.response)
                self.get_url_key(self.response)

            except Exception as e: 
                print(e)
        
            #print(self.urls)


    def add_headers(self, request_obj):

        request_obj.add_header('User-Agent',self.USER_AGENT)
        request_obj.add_header('Cookie', self.cookies)
        request_obj.add_header('referer', 'https://notepad.pw/'+self.url_key)
        request_obj.add_header('x-requested-with', 'XMLHttpRequest')
        request_obj.add_header('accept-encoding', 'gzip, deflate, br')
        
        return request_obj



    def hit(self, url):

        request_obj = Request(url)
        request_obj = self.add_headers(request_obj)

        response_obj = request.urlopen(request_obj)
        self.cookies = response_obj.getheader('Set-Cookie')
        self.content = decompress(response_obj.read()).decode('utf-8')

        return


    def get_pad_key(self, response):

        pad_key_pattern = r'(?<=pad_key = \')[a-z0-9]+'
        results = re.findall(pad_key_pattern, response)

        if(results):

            for pad_key in results:
                self.pad_key = pad_key

    def get_url_key(self, response):

        url_key_pattern = r'(?<=url_key = \')[a-z0-9]+'
        results = re.findall(url_key_pattern, response)

        if(results):

            for url_key in results:
                if(url_key):
                    self.url_key = url_key


    def save(self, data):


        encoded_data = urlencode(data).encode('utf-8')

        request_obj = Request(self.save_url, method="POST", data=encoded_data, headers={})

        request_obj = self.add_headers(request_obj)
        request_obj.add_header('Content-Type', 'application/x-www-form-urlencoded; charset=UTF-8')
        request_obj.add_header('origin', 'https://notepad.pw')

        response_obj = request.urlopen(request_obj)




if __name__ == '__main__':
    content = ""
    #url = input('Enter the url : ')
    url = "https://notepad.pw/chaityashah"
    spider = Spider(url)

    print(spider.url_key)
    print(spider.pad_key)






from npwcli import __version__

import os
import sys

try:
	from setuptools import setup
except ImportError:
	from distutils.core import setup


dependencies = ['docopt', 'termcolor', 'socketIO_client', 'brotli']


def publish():
	os.system("python3 setup.py sdist upload")



if sys.argv[-1] == "publish":
	publish()
	sys.exit()


setup(
	name='npw',
	version='.'.join(str(i) for i in __version__),
	description='A notepad.pw magician in your command line',
	url='https://github.com/Chaitya62/notepadpw-cli',
	author='Chaitya Shah',
	author_email='chaitya.shah@somaiya.edu',
	install_requires=dependencies,
	packages=['npwcli'],
	entry_points={
		'console_scripts': [
			'npw=npwcli.cli:start'
		]
	},
	classifiers=[
		'Development Status :: 4 - Beta',
		'Intended Audience :: Developers',
		'Natural Language :: English',
		'Programming Language :: Python',
	]
	) 


